import chimera, numpy as np, Rotamers, Tkinter as tk, tkMessageBox
from chimera import runCommand, selection
from chimera.baseDialog import ModelessDialog
from Rotamers import getRotamers, useRotamer
import pickle

from StructBio.EnergyFunctions.LJ import LJ_evaluation
from StructBio.Plots.surfacePlotBundle import SurfacePlotBundle
from StructBio.Utilities.shellNeighbors import Shell


# Dimensions of canvas and limits on y-values:
CHI_A_DEG_INCR = 3
CHI_B_DEG_INCR = 6
CANVAS_H = 365      # Canvas plot height
CANVAS_W = 800      # Canvas plot width
Y_MIN = -22.        # Minimum expected y value.
Y_MAX = 102.        # Maximum expected y value.
# Lists for horizontal grid lines
GRID_DARK_L = [0.0] 
GRID_GRAY_L = [100., 90., 80., 70., 60., 50., 40., 30., 20., 10., -10., -20.]

Z_MAX = 100.0        # Maximum value of zV_A for eventual display via Save button.
        
class sideChainControlPanel(ModelessDialog):
    buttons = ("Close")
    help = False    # Leave this out to get a Help button.
    title = "Side Chain Control Panel"

    def fillInUI(self, parent):
        self.res = None
        superFrameLeft = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        superFrameLeft.grid(row = 0, column = 0, columnspan = 5, padx = 4, pady = 4, sticky = tk.W)
        superFrameLeft.grid_columnconfigure(0, weight = 1)

        self.superFrameRight = tk.Frame(parent, bd = 2, relief = tk.RIDGE) 
        self.superFrameRight.grid(row = 0, column = 6, columnspan = 15, padx = 4, pady = 4, sticky = tk.W)
        self.superFrameRight.grid_columnconfigure(0, weight = 1)
        # See stripChart button function for widgets that go into this frame.
        
        #-------------------------------------------------------------------------
        # SUBFRAME 0
        # Set up the salutation, Accept button and residue information Label:
        acceptFrame = tk.Frame(superFrameLeft, bd = 2, relief = tk.RIDGE)
        acceptFrame.grid(row = 0, column = 0, columnspan = 3, padx = 4, pady = 4, sticky = tk.W + tk.E)
        acceptFrame.grid_columnconfigure(0, weight = 1)
        
        # Text to be displayed at the top of the dialog window:
        topText = tk.Label(acceptFrame, text =  " Select an atom to choose a side-chain. Then click Accept.",
                           font = ("Helvetica", 11, "normal"))
        topText.grid(row = 0, column = 0, columnspan = 4, pady = 4, sticky = tk.W)

        # Button to accept atom that specifies a side chain:
        self.acceptButton = tk.Button(acceptFrame, text = " Accept ", command = self.acceptSelection)
        self.acceptButton.grid(row = 1, column = 1, padx = 4, pady = 4, sticky = tk.E)

        # Label to describe the chosen residue:
        self.resInfoVar = tk.StringVar()
        resInfoLabel = tk.Label(acceptFrame, textvariable = self.resInfoVar, width = 10,
                               bg = "white", bd = 2, relief = tk.GROOVE)
        resInfoLabel.grid(row = 1, column = 2, padx = 4, pady = 4, sticky = tk.E)

        #-------------------------------------------------------------------------
        # SUBFRAME 1
        # Setting up the scales:
        scaleFrame = tk.Frame(superFrameLeft, bd = 2, relief = tk.RIDGE)
        scaleFrame.grid(row = 1, column = 0, columnspan = 3, padx = 4, pady = 4, sticky = tk.W + tk.E)
        
        scaleText = tk.Label(scaleFrame, text =  " Torsion angles for this side chain:  ")
        scaleText.grid(row = 0, column = 0, columnspan = 2, pady = 2, sticky = tk.W)

        # Declare all dictionaries.  Keys will be torsion angles:
        keys_L = ["chi1", "chi2", "chi3", "chi4"]
        self.scaleVar_D = {}; self.scale_D = {}; self.chiOutVar_D = {}; chiOut_D = {}; self.increasing_D = {}
        self.initialChiVals_D = {}

        for k in keys_L:
            self.increasing_D[k] = True
            
            # Label with name of torsion angle:
            tk.Label(scaleFrame, text = " " + k + ": ", font = ("Helvetica", 11, "normal")) \
                                 .grid(row = int(k[3]), column = 0, pady = 2, sticky = tk.W)
            # Scale:
            self.scaleVar_D[k] = tk.DoubleVar()
            self.scale_D[k] = tk.Scale(scaleFrame, variable = self.scaleVar_D[k], orient = tk.HORIZONTAL,
                                       command = lambda event, arg = k: self.updateValue(arg),
                                       length = 360, from_ = 0.0, to = 360.0, resolution = 0.1,
                                       troughcolor = "gray", showvalue = False)
            self.scale_D[k].grid(row = int(k[3]), column = 1, sticky = tk.W)
            self.scale_D[k].set(180.0)
            self.scale_D[k].config(state = tk.DISABLED)

            # Label that displays the chi angle:
            self.chiOutVar_D[k] = tk.StringVar()
            chiOut_D[k] = tk.Label(scaleFrame, textvariable = self.chiOutVar_D[k], width = 6,
                                   bg = "white", bd = 2, relief = tk.GROOVE)
            chiOut_D[k].grid(row = int(k[3]), column = 2, padx = 4, pady = 4, sticky = tk.E)

        #-------------------------------------------------------------------------
        # SUBFRAME 2
        # Label, animation buttons and Save button:
        animationFrame = tk.Frame(superFrameLeft, bd = 2, relief = tk.RIDGE)
        animationFrame.grid(row = 2, column = 0, columnspan = 3, padx = 4, pady = 4, sticky = tk.W + tk.E)
        animationFrame.grid_columnconfigure(0, weight = 1)
        
        tk.Label(animationFrame, text =  " Animation:  ") \
                         .grid(row = 0, column = 0, pady = 2, sticky = tk.W)

        self.chi1chi2AnimateButton = tk.Button(animationFrame, text = " Chi1 & Chi2 ", state = tk.DISABLED,
                                               command = lambda: self.animate2TorsionAngles("chi1", "chi2"))
        self.chi1chi2AnimateButton.grid(row = 0, column = 1, padx = 2, pady = 4, sticky = tk.E)

        self.chi2chi3AnimateButton = tk.Button(animationFrame, text = " Chi2 & Chi3 ", state = tk.DISABLED,
                                               command = lambda: self.animate2TorsionAngles("chi2", "chi3"))
        self.chi2chi3AnimateButton.grid(row = 0, column = 2, padx = 2, pady = 4, sticky = tk.E)

        self.chi3chi4AnimateButton = tk.Button(animationFrame, text = " Chi3 & Chi4 ", state = tk.DISABLED,
                                               command = lambda: self.animate2TorsionAngles("chi3", "chi4"))
        self.chi3chi4AnimateButton.grid(row = 0, column = 3, padx = 2, pady = 4, sticky = tk.E)

        self.saveButton = tk.Button(animationFrame, text = " Save ", state = tk.DISABLED, command = self.saveBtnFunc)
        self.saveButton.grid(row = 0, column = 4, padx = 6, pady = 4, sticky = tk.E)

        
        #-------------------------------------------------------------------------
        # SUBFRAME 3
        # Label and Spinbox to allow the user to choose a rotamer:
        spinBoxFrame = tk.Frame(superFrameLeft, bd = 2, relief = tk.RIDGE)
        spinBoxFrame.grid(row = 3, column = 0, columnspan = 3, padx = 4, pady = 4, sticky = tk.W + tk.E)
        spinBoxFrame.grid_columnconfigure(0, weight = 1)
        
        sbLabel = tk.Label(spinBoxFrame, text =  " Choose a rotamer:  ")
        sbLabel.grid(row = 0, column = 0, columnspan = 2, padx = 4, pady = 4, sticky = tk.E)

        self.spinBoxVar = tk.StringVar()
        self.spinBox = tk.Spinbox(spinBoxFrame, textvariable = self.spinBoxVar,
                                  command = self.spinBoxFunc, state = 'readonly', wrap = True)
        self.spinBox.grid(row = 0, column = 3, padx = 4, pady = 4, sticky = tk.E)
        self.spinBox.configure(state = tk.DISABLED)

    
        #-------------------------------------------------------------------------
        # SUBFRAME 4
        # Label and Button to reset torsions to initial values:
        resetBtnFrame = tk.Frame(superFrameLeft, bd = 2, relief = tk.RIDGE)
        resetBtnFrame.grid(row = 4, column = 0, columnspan = 3, padx = 4, pady = 4, sticky = tk.W + tk.E)
        resetBtnFrame.grid_columnconfigure(0, weight = 1)
        
        tk.Label(resetBtnFrame, text =  " Reset torsion angles to initial values:  ") \
                         .grid(row = 0, column = 0, columnspan = 2, pady = 2, sticky = tk.E)

        self.resetButton = tk.Button(resetBtnFrame, text = " Reset ", state = tk.DISABLED,
                                     command = self.resetTorsionAngles)
        self.resetButton.grid(row = 0, column = 3, padx = 2, pady = 4, sticky = tk.E)

        #-------------------------------------------------------------------------
        # SUBFRAME 5
        # Label to give energy value:        
        energyFrame = tk.Frame(superFrameLeft, bd = 2, relief = tk.RIDGE)
        energyFrame.grid(row = 5, column = 0, columnspan = 3, padx = 4, pady = 4, sticky = tk.W + tk.E)
        energyFrame.grid_columnconfigure(0, weight = 1)
        
        # Text to be displayed prior to energy label:
        energyText = tk.Label(energyFrame, text =  " Lennard-Jones energy value: ")
        energyText.grid(row = 0, column = 0, columnspan = 2, pady = 4, sticky = tk.E)

        # Label giving energy value:
        self.energyInfoVar = tk.StringVar()
        self.energyInfoLabel = tk.Label(energyFrame, textvariable = self.energyInfoVar, width = 16,
                                        bg = "white", bd = 2, relief = tk.GROOVE)
        self.energyInfoLabel.grid(row = 0, column = 3, padx = 4, pady = 4, sticky = tk.E)


        #-------------------------------------------------------------------------
        # Set up canvas for strip chart:
        canvasFrame = tk.Frame(self.superFrameRight, bd = 2, relief = tk.RIDGE)
        canvasFrame.grid(row = 3, column = 0, padx = 4, pady = 4, sticky = tk.E)
        canvasFrame.grid_rowconfigure(0, weight = 1)
        canvasFrame.grid_columnconfigure(0, weight = 1)

        yAxis = tk.Canvas(canvasFrame, width = 60,
                          height = CANVAS_H, bg = "white")
        yAxis.grid(row = 0, column = 0, pady = 4, sticky = tk.N + tk.S)
        
        self.stripChart = tk.Canvas(canvasFrame, width = CANVAS_W,
                                   height = CANVAS_H, bg = "white")
        self.stripChart.grid(row = 0, column = 1, pady = 4, sticky = tk.N + tk.S)
        
        hbar = tk.Scrollbar(canvasFrame, orient = tk.HORIZONTAL,
                            command = self.stripChart.xview)
        hbar.grid(row = 1, column = 1, sticky = tk.E + tk.W)
        self.stripChart.configure(xscrollcommand = hbar.set)
        #self.stripChart.configure(xscrollincrement = 1)

        #-------------------------------------------------------------------------
        # Set up Y-axis in first canvas:
        yAxis.create_line(50, self.scaleYval(Y_MIN),
                          50, self.scaleYval(Y_MAX), width = 3)
        
        # Draw ticks:
        for t in GRID_DARK_L:
            yAxis.create_line(45, self.scaleYval(t),
                              60, self.scaleYval(t), width = 2)
            yAxis.create_text(40, self.scaleYval(t), text = '%5.1f' % t, anchor = tk.E)
        for t in GRID_GRAY_L:
            yAxis.create_line(45, self.scaleYval(t),
                              60, self.scaleYval(t), width = 3, fill = "GRAY")        
            yAxis.create_text(40, self.scaleYval(t), text = '%5.1f' % t, anchor = tk.E)
            



        #-------------------------------------------------------------------------
        # Frames are done.  Set up energy evaluator:
        self.energyEvaluator = LJ_evaluation()


        
    #=============================================================================
    # CALLBACK functions:
    #=============================================================================
    # SUBFRAME 0 callbacks: --------------------------------------------------
    def acceptSelection(self):
        self.stripChart.delete(tk.ALL) # Clear out any previously drawn strip chart.
        self.energyInfoVar.set("")
        
        atm_L = selection.currentAtoms()
        # If there is no selection everything goes back to the initial state:
        if len(atm_L) == 0:
            map(self.disableScale, ("chi1", "chi2", "chi3", "chi4"))
            self.resInfoVar.set("")
            self.resetButton.configure(state= tk.DISABLED)
            self.initialChiVals_D = {}
            self.spinBox.configure(values = [" "])
            self.spinBox.configure(state = tk.DISABLED)
            return
        
        # If we get this far, we have a residue.
        self.resetButton.configure(state= tk.NORMAL)
        self.res = atm_L[0].residue

        # Remember chi angles:
        if self.res.chi1 != None: self.initialChiVals_D["chi1"] = self.res.chi1           
        if self.res.chi2 != None: self.initialChiVals_D["chi2"] = self.res.chi2
        if self.res.chi3 != None: self.initialChiVals_D["chi3"] = self.res.chi3
        if self.res.chi4 != None: self.initialChiVals_D["chi4"] = self.res.chi4        

        # Get rotamer list and populate spinbox:
        self.rots_L = []
        if not self.res.type in ("ALA", "GLY"):
            (flag, self.rots_L) = getRotamers(self.res)
        else:
            self.spinBox.configure(values = [" "])
        self.spinBoxVals_L = []
        for ix in range(len(self.rots_L)):
            self.spinBoxVals_L.append("%2d:  P = %12.10f" % (ix, self.rots_L[ix].rotamerProb))

        if len(self.rots_L) > 0:
            self.spinBox.configure(state = tk.NORMAL)
            self.spinBox.configure(values = self.spinBoxVals_L)
        else:
            self.spinBox.configure(state = tk.DISABLED)

        
        runCommand("sel :" + str(self.res.id.position) + "." +
                   self.res.id.chainId + " za<8.0; focus sel; ~sel")
        self.resInfoVar.set(self.res.type + " " + str(self.res.id.position) + "." + self.res.id.chainId)

        # Prepare shell for later use:
        atoms_L = []    # Get a list of all atoms not in this residue:
        for a in self.res.molecule.atoms:
            if not a in self.res.atoms: atoms_L.append(a)
        self.shell = Shell(atoms_L, 0.0, 10.0)

        self.setUpScales()

        if sideChainInCycle(self.res):
            tkMessageBox.showwarning("NOTE: ", "You cannot change torsion angles \n for side chains in a cycle.")

        self.displayEnergy()

    # -----------------------------------------------------------------------
    def setUpScales(self):
        # Set up scales for user interaction and while doing this enable the appropriate buttons in the
        # animation frame.
        self.saveButton.configure(state = tk.DISABLED)
        self.chi1chi2AnimateButton.configure(state= tk.DISABLED)
        self.chi2chi3AnimateButton.configure(state= tk.DISABLED)
        self.chi3chi4AnimateButton.configure(state= tk.DISABLED)
        
        if self.res.chi1 != None: self.enableScale("chi1", self.res.chi1)
        else: self.disableScale("chi1")
        
        if self.res.chi2 != None:
            self.enableScale("chi2", self.res.chi2)
            self.chi1chi2AnimateButton.configure(state= tk.NORMAL)
        else: self.disableScale("chi2")
        
        if self.res.chi3 != None:
            self.enableScale("chi3", self.res.chi3)
            self.chi2chi3AnimateButton.configure(state= tk.NORMAL)
        else: self.disableScale("chi3")
        
        if self.res.chi4 != None:
            self.enableScale("chi4", self.res.chi4)
            self.chi3chi4AnimateButton.configure(state= tk.NORMAL)
        else: self.disableScale("chi4")

    def enableScale(self, arg, chiVal):
        self.scaleVar_D[arg].set(180.0 + chiVal)                # Set scale to continue with chi value.
        self.increasing_D[arg] = True                           # Initialize to " values increasing" mode.
        self.chiOutVar_D[arg].set("%7.1f " % chiVal)            # Initialize scale Label.
        if not sideChainInCycle(self.res):                      # Chimera disallows changing torsions if sidechain is part of cycle.
            self.scale_D[arg].config(state = tk.NORMAL)         # Enable it for user interaction.
            self.scale_D[arg].config(troughcolor = "goldenrod1")# Change color to indicate being enabled.
        else:
            self.scale_D[arg].config(state = tk.DISABLED)       # Enable it for user interaction.
            self.scale_D[arg].config(troughcolor = "gray")      # Change color to indicate being enabled.
            self.resetButton.configure(state= tk.DISABLED)

    def disableScale(self, arg):
        self.scaleVar_D[arg].set(180.0)                         # Initialize scale.
        self.scale_D[arg].config(state = tk.DISABLED)           # Disable the scale.
        self.scale_D[arg].config(troughcolor = "gray")          # Change color to indicate being dusabled.
        self.increasing_D[arg] = True                           # Initialize to " values increasing" mode.
        self.chiOutVar_D[arg].set("")                           # Clear scale Label.

        
    # SUBFRAME 1 callbacks  --------------------------------------------------
    # needed for Scale widget:
    def updateValue(self, arg):
        if not self.res: return
        value = self.scaleVar_D[arg].get()
        if value == 0.0 or value == 360: self.increasing_D[arg] = not self.increasing_D[arg]
        if self.increasing_D[arg]:
            self.chiOutVar_D[arg].set("%7.1f " % (value - 180.0))
            self.changeChi(arg, value - 180.0)
        if not self.increasing_D[arg]:
            self.chiOutVar_D[arg].set("%7.1f " % (180.0 - value))
            self.changeChi(arg, 180.0 - value)
        self.displayEnergy()
        
    def changeChi(self, arg, newVal):
        if arg == "chi1": self.res.chi1 = newVal
        if arg == "chi2": self.res.chi2 = newVal
        if arg == "chi3": self.res.chi3 = newVal
        if arg == "chi4": self.res.chi4 = newVal

    # SUBFRAME 2 callbacks  --------------------------------------------------
    # needed for animation buttons:
    def animate2TorsionAngles(self, torsionA, torsionB):
        self.torsionA = torsionA; self.torsionB = torsionB
        self.stripChart.delete(tk.ALL)
        # If the next "useless" line is removed, the grid lines and the tick lines
        # fall out of alignment.  Send me email if you have a better way to avoid this kludge.
        self.stripChart.create_line(50, self.scaleYval(Y_MIN),
                                    50, self.scaleYval(Y_MAX), width = 3, fill = "white")
        
        # Disable other widgets while animation is executing:
        self.spinBox.configure(state = tk.DISABLED)
        self.resetButton.configure(state = tk.DISABLED)
        self.acceptButton.configure(state = tk.DISABLED)

        self.numTorsionA = 360/CHI_A_DEG_INCR
        self.numTorsionB = 360/CHI_B_DEG_INCR

        # Declare arrays to be used by Save button:
        # We will evaluate from -180 to +180 INCLUSIVE.
        m = self.numTorsionA + 1
        n = self.numTorsionB + 1
        self.x_A = np.zeros((m,n), float)
        self.y_A = np.zeros((m,n), float)
        self.zV_A = np.zeros((m,n), float)

        countForGridDraw = 0
        for ix in range(m):
            chiAVal = -180.0 + ix*CHI_A_DEG_INCR
            self.changeChi(self.torsionA, chiAVal)
            self.enableScale(self.torsionA, chiAVal)
            for jx in range(n):
                chiBVal = -180.0 + jx*CHI_B_DEG_INCR
                self.changeChi(self.torsionB, chiBVal)
                self.enableScale(self.torsionB, chiBVal)
                energyValue = self.displayEnergy()
                
                # Storing for Save button function:
                self.x_A[ix, jx] = chiAVal
                self.y_A[ix, jx] = chiBVal
                self.zV_A[ix,jx] = min(Z_MAX, 10.* energyValue) # Do not display energy values over Z_MAX.
                # NOTE: We are displaying energy value MULTIPLIED BY TEN to give better resolution for
                # the low evergies.
                
                if ix == 0 and jx == 0:
                    firstPt = energyValue
                    self.prevPoint = self.scaleYval(max(min(Y_MAX, firstPt), Y_MIN))
                else:
                    self.update(self.scaleYval(max(min(Y_MAX, energyValue), Y_MIN)))
                    self.stripChart.update()
                    countForGridDraw += 1
                    if countForGridDraw%10 == 1:
                        for g in GRID_DARK_L:
                            self.stripChart.create_line(1, self.scaleYval(g),
                                                       -9, self.scaleYval(g), width = 2)
                        for g in GRID_GRAY_L:
                            self.stripChart.create_line(1, self.scaleYval(g),
                                                       -9, self.scaleYval(g),
                                                       width = 1, fill = "GRAY")

        self.saveButton.configure(state = tk.NORMAL)
        # Recover functionality of the other widgets:
        self.spinBox.configure(state = tk.NORMAL)
        self.resetButton.configure(state= tk.NORMAL)
        self.acceptButton.configure(state= tk.NORMAL)


    def saveBtnFunc(self):
        zMin = min(map(min, self.zV_A))
        zMax = max(map(max, self.zV_A))
        zRange = zMax - zMin

        (m, n) = self.zV_A.shape
        colors_A = np.zeros((m,n, 3), float)
        transparency_A = np.zeros((m, n), float)

        colorBlue = np.array((0.0, 0.0, 1.0))
        colorRed = np.array((1.0, 0.0, 0.0))
        colorRange = colorRed - colorBlue
        
        for i in range(m):
            for j in range(n):
                colors_A[i,j,:] = colorBlue + colorRange*(self.zV_A[i,j] - zMin)/zRange
                transparency_A[i, j] = 1.0
                if self.zV_A[i, j] == Z_MAX: transparency_A[i, j] = 0.0
                

        spb = SurfacePlotBundle(self.x_A, self.y_A, self.zV_A,
                                color = colors_A, transparency = transparency_A)

        fileName = (self.res.molecule.name + "_" + self.res.type + "_" +
                    str(self.res.id.position) + "_" + self.res.id.chainId + ".pkl")

        fileHandler = open(fileName, "wb")
        pickle.dump(spb, fileHandler, -1)
        fileHandler.close()

        self.saveButton.configure(state = tk.DISABLED)
        
    #-------------------------------------------------------------------------
    # Function to insert new point in stripChart:
    def update(self, newPoint):
        self.stripChart.move(tk.ALL, 1, 0)
        self.stripChart.create_line(0, newPoint, 1, self.prevPoint, width = 2)
        self.prevPoint = newPoint
        self.stripChart.configure(scrollregion = self.stripChart.bbox(tk.ALL))

    #-------------------------------------------------------------------------
    # Function to convert a real y value to its corresponding vertical offset
    # on the canvas:            
    def scaleYval(self, nextY):
        return CANVAS_H - int(CANVAS_H*(nextY - Y_MIN)/(Y_MAX - Y_MIN))


    # SUBFRAME 3 callbacks  --------------------------------------------------
    # needed for spin box widget:
    def spinBoxFunc(self):
        spinVal = self.spinBoxVar.get()
        rotIx = int(spinVal.split(":")[0])
        useRotamer(self.res, [self.rots_L[rotIx]])
        self.setUpScales()
        self.displayEnergy()
    
    # SUBFRAME 4 callbacks  --------------------------------------------------
    # needed for Reset button:
    def resetTorsionAngles(self):
        if sideChainInCycle(self.res): return
        if self.res.chi1 != None:
            self.enableScale("chi1", self.initialChiVals_D["chi1"])
            self.res.chi1 = self.initialChiVals_D["chi1"]
        if self.res.chi2 != None:
            self.enableScale("chi2", self.initialChiVals_D["chi2"])
            self.res.chi2 = self.initialChiVals_D["chi2"]
        if self.res.chi3 != None:
            self.enableScale("chi3", self.initialChiVals_D["chi3"])
            self.res.chi3 = self.initialChiVals_D["chi3"]
        if self.res.chi4 != None:
            self.enableScale("chi4", self.initialChiVals_D["chi4"])
            self.res.chi4 = self.initialChiVals_D["chi4"]
        self.displayEnergy()

    # SUBFRAME 5 callbacks  --------------------------------------------------
    # needed for energy output label:
    def displayEnergy(self):
        energy = 0.0
        for atm in self.res.atoms:
            atmNbrs = self.shell.getAtomsInShell(atm)
            for nbr in atmNbrs:
                # Skip residues that cannot be handled by the LJ evalutor:
                # This avoids a key error but it means that a residue near a non standard
                # residue should not be selected otherwise the energy is not properly calculated.
                if nbr.residue.type in ("ALA", "ARG", "ASN", "ASP", "CYS", "GLU", "GLN",
                                        "GLY", "HIS", "ILE", "LEU", "LYS", "MET", "PHE",
                                        "PRO", "SER", "THR", "TRP", "TYR", "VAL"):
                    energy += self.energyEvaluator.atomPairEnergy(nbr, atm)
                
        self.energyInfoVar.set("%8.2f " % energy)
        return energy
    

#-------------------------------------------------------------------------        
def sideChainInCycle(r):
    if r.type == "PRO": return True
    if r.type == "CYS" and "SG" in [a.name for a in r.findAtom("SG").neighbors]:
        return True
    return False
    
#===============================================================================
# Mainline for testing:        
wb = sideChainControlPanel()
